function [fcalls, error] = oscError(ode, mtd, hPow, solExact, hExact, varargin)
% OSCERROR calculates and plots error in mathematical pendulum, compared to
% solution solExact.

% created by Claude Gittelson

n = length(hPow);
fcalls = zeros(1,n);
error = zeros(1,n);
h = 2.^hPow;

for i=1:n
    sol = mtd.solve(ode, h(i));
    fcalls(i) = sol.fcalls;
    hstep = h(i)/hExact;
    for k=1:length(sol.t)
        e = norm(sol.y(:,k)-solExact.y(:,1+hstep*(k-1)));
        error(i) = max(e, error(i));
    end
end

loglog(fcalls, error, varargin{:});
hold on;
xlabel('function evaluations');
ylabel('max. error');
axis([10 10^5 10^-13 10]);