function [y, fcalls] = rkDIStep(param, ode, h, tol, y1, t, k, fcalls)
%RKDISTEP step of diagonally implicit runge kutta method

% created by Claude Gittelson

lastk = k(:,param.s);
for i = 1:param.s
    constT = t - h + h*param.c(i);
    constY = y1 + h*k*(param.A(i,:)');
    if(param.explicit(i))
        k(:,i) = ode.f(constT, constY, ode.fargs{:});
        lastk = k(:,i);
        fcalls = fcalls + ode.dim;
    else            
        % newton iteration
        x(:,1) = lastk;
        d = zeros(ode.dim,1);
		m = 1;
		u = 2; v = 1; w = 1;
        [L,U,P] = lu(eye(ode.dim) - h*param.A(i,i)*ode.Df(constT, constY + h*param.A(i,i)*x(:,v), ode.fargs{:}));
		while (norm(d)>tol | m==1) & m<100 
			g = x(:,v) - ode.f(constT, constY + h*param.A(i,i)*x(:,v), ode.fargs{:});
            if rem(m,10) == 0
                [L,U,P] = lu(eye(ode.dim) - h*param.A(i,i)*ode.Df(constT, constY + h*param.A(i,i)*x(:,v), ode.fargs{:}));
                fcalls = fcalls + ode.dim^2;
            end
            d = U\(L\(P*(-g)));
			x(:,u) = x(:,v) + d;
			w = u; u = v; v = w; m = m+1;
		end
        k(:,i) = x(:,w);
        lastk = k(:,i);
        fcalls = fcalls + (m-1)*ode.dim + ode.dim^2;
    end
end
y = y1 + h*k*param.b;
