function param = rkParams(c, A, b, name)
%RKPARAMS create parameters for rk. PARAM = RKPARAMS(C,A,B).

% created by Claude Gittelson

if(nargin <= 2)
    return;
end

param.s = length(b); % number of steps
% check dimensions
if(length(c) < param.s | size(A) ~= [param.s, param.s])
    return;
end

% b needs to be a column vector
[m, n] = size(b);
if(m ~= param.s)
    b = b';
end

param.c = c;
param.A = A;
param.b = b;
param.name = '';
if(nargin>=4)
    if(ischar(name))
        param.name = name;
    end
end

% check if method is explicit
E = triu(A);
param.explicit = zeros(1, param.s);
for i=1:param.s
    param.explicit(i) = (norm(E(i,:))==0);
end

% check if method is diagonally implicit
x = 0;
for i=1:param.s
    for j=i+1:param.s
        x = x + A(i,j);
    end
end
param.dirk = (x==0);

% implementation of adjoint
% syntax: adjointMtd = mtd.adjoint(mtd);

param.adjoint = @(varargin) rkAdjoint(param, varargin{:});

% implementation of rk solver
if param.dirk
    param.step = @rkDIStep;
else
    param.step = @rkFullStep;
end

param.solver = @rk;

param.solve = @(varargin) rk(param, varargin{:});
