function param = rkSplitParams(flow, scheme, weight, name)
%RKSPLITPARAMS create parameters for split rk method
% flow is a cell arry of function handles for individual compontets of the
% vector field.

% created by Claude Gittelson

if(length(scheme) ~= length(weight) | nargin < 2)
    return;
end

param.stages = length(scheme); % number of stages in composition

param.flow = flow; % vector of flows of parts of vector field
param.scheme = scheme; % vector of form [2 1] for lie trotter, [1 2 1] for strang
param.weight = weight; % vector of form [1 1] for lie trotter, [0.5 1 0.5] for strang

param.s = 1; % one-stage runge kutta method

param.name = 'Splitting Method';
if(nargin>=4)
    if(ischar(name))
        param.name = name;
    end
end

% implementation of adjoint
% syntax: adjointMtd = mtd.adjoint(mtd);

param.adjoint = @(varargin) rkSplitAdjoint(param, varargin{:});

% implementation of rk solver
param.step = @rkSplitStep;
param.solver = @rk;
param.solve = @(varargin) rk(param, varargin{:});
