function S = varAction(mtd, ode, Lh, h, varSize)
%VARACTION calculates variations of discrete action

% created by Claude Gittelson

% set default size of variation
if nargin <= 4
    varSize = 2e-5;
end

% solve ode
sol = mtd.solve(ode, h);
q = sol.y(1,:);
N = length(q)-1;

% calculate (cubic) variations of q
x = [-1:2/N:1];
x2 = (x-1).*(x+1);
r = [-32*varSize:varSize:32*varSize]';
dq = zeros(length(r)^2, length(x));
for a=1:length(r)
    for b=1:length(r)
        dq(length(r)*(a-1)+b,:) = x2.*(r(b)*x+r(a));
    end
end

% calculate discrete action for variations and solution
ds = Lh(q(1:N),q(2:N+1));
dS = zeros(size(dq,1), length(x)-1);
Q = zeros(length(x),1);
for i=1:size(dq,1)
    Q(:) = q + dq(i,:);
    dS(i,:) = Lh(Q(1:N),Q(2:N+1));
end

s = sum(ds);
S = sum(dS')-s;

% plot results :
% num. solution is extremum of action

n = [1:length(r)];
indx = n(ones(length(r),1),:);
n = n';
indy = n(:,ones(length(r),1));
ind = [indy(:),indx(:)];
A = accumarray(ind, S);
figure;
surf(r,r,A);
colormap(jet);
shading interp;
hold on;
plot(0,0,'k.');
plot(0,0,'ro');

% figure;
% Y = r(:,ones(length(r),1));
% r = r';
% X = r(ones(length(r),1),:);
% plot3(X(:), Y(:), S', 'k.');
% hold on;
% plot(0, 0, 'ro');

s_min = min(S);
s_max = max(S);
r_min = min(r);
r_max = max(r);
axis([r_min r_max r_min r_max s_min s_max]);

string = sprintf('Discrete Action of %s for %s', ode.name, mtd.name);
title(string);
xlabel('a');
ylabel('b');
zlabel('Discrete Action S');
